package com.easefun.polyvsdk;

import android.content.Context;
import android.support.annotation.NonNull;

/**
 * 下载接口
 * @author Lion 2016-12-13
 */
public interface IPolyvDownloader {
    /**
     * 是否在下载中
     * @return 下载中返回true
     */
    boolean isDownloading();

    /**
     * 开始下载
     */
    void start();

    /**
     * 开始下载
     * @param context 设置此参数会判断SDK卡权限是否拥有
     */
    void start(Context context);

    /**
     * 停止下载<br/>
     * 已下载完成的资源会保留成文件
     */
    void stop();

    /**
     * 停止下载<br/>
     * 已下载完成的资源会保留成文件
     * @param isDestroy - 是否销毁
     */
    void stop(boolean isDestroy);

    /**
     * 当下载是下载m3u8视频，ts文件存在时，是否回调进度。
     * @param isCallback 默认为true
     */
    void isCallbackProgressWhereExists(boolean isCallback);

    /**
     * 获取当下载是下载m3u8视频，ts文件存在时是否回调进度。
     */
    boolean isCallbackProgressWhereExists();

    /**
     * 设置速度回调间隔
     * @param interval 毫秒，默认值2000
     */
    void setSpeedCallbackInterval(int interval);

    /**
     * 取得速度回调间隔时间
     * @return 毫秒
     */
    int getSpeedCallbackInterval();

    /**
     * 取得在使用的context
     * @return The context to use
     */
    Context getContext();

    /**
     * 删除视频文件<br/>
     * 删除之前会做stop停止动作
     * @return 固定返回true
     */
    boolean deleteVideo();

    /**
     * 删除视频文件<br/>
     * 删除之前会做stop停止动作
     * @param videoId 视频id
     * @param bitRate 码率（清晰度）
     * @return 固定返回true
     * @see #deleteVideo()
     */
    @Deprecated
    boolean deleteVideo(@NonNull String videoId, int bitRate);

    /**
     * 删除视频文件<br/>
     * 删除之前会做stop停止动作
     * @param videoId 视频id
     * @param bitRate 码率（清晰度）
     * @param hlsSpeedType 播放速度类型 {@link Video.HlsSpeedType}，非加密视频设置此参数没有效果，默认值{@link Video.HlsSpeedType#SPEED_1X}。<b>请使用{@link com.easefun.polyvsdk.video.PolyvVideoView#setSpeed(float)}设置播放速度</b>
     * @return 固定返回true
     * @see #deleteVideo()
     */
    @Deprecated
    boolean deleteVideo(@NonNull String videoId, int bitRate ,@NonNull Video.HlsSpeedType hlsSpeedType);
}
