package com.easefun.polyvsdk;

/**
 * 下载失败原因
 * @author Lion 2016-2-24
 */
public class PolyvDownloaderErrorReason {
	/** 失败类型 */
	private final ErrorType type;
	/** 错误 */
	private final Throwable cause;

	public PolyvDownloaderErrorReason(ErrorType type, Throwable cause) {
		this.type = type;
		this.cause = cause;
	}

	public ErrorType getType() {
		return type;
	}

	@SuppressWarnings("unused")
	public Throwable getCause() {
		return cause;
	}
	
	/**
	 * 错误类型
	 * @author TanQu
	 */
	public enum ErrorType {
		/** vid为null */
		VID_IS_NULL(10001),
		/** 没有权限访问视频 */
		NOT_PERMISSION(10002),
		/** 运行时异常 */
		RUNTIME_EXCEPTION(10003),
		/** 视频状态错误 */
		VIDEO_STATUS_ERROR(10004),
		/** m3u8没有数据 */
		M3U8_NOT_DATA(10005),
		/** 问答没有数据 */
		QUESTION_NOT_DATA(10006),
		/** ts下载列表为空 */
		MULTIMEDIA_LIST_EMPTY(10007),
		/** 不能创建文件夹 */
		CAN_NOT_MKDIR(10008),
		/** 下载ts错误 */
		DOWNLOAD_TS_ERROR(10009),
		/** 下载地址为空 */
		MULTIMEDIA_EMPTY(10010),
		/** 不能创建目录 */
		NOT_CREATE_DIR(10011),
		/** video加载失败，不会再回调该异常 */
		@Deprecated
		VIDEO_LOAD_FAILURE(10012),
		/** video取得为null */
		VIDEO_NULL(10013),
		/** 没有设置下载文件夹 */
		DOWNLOAD_DIR_IS_NUll(10014),
		/** 视频速度为null */
		HLS_SPEED_TYPE_IS_NULL(10015),
		/** 1.5倍下载地址为空 */
		HLS_15X_URL_ERROR(10016),
		/** 存储空间不足 */
		DIR_SPACE_LACK(10017),
		/** 不支持1.5倍 */
		HLS_15X_ERROR(10018),
		/** 取得视频信息错误 */
		GET_VIDEO_INFO_ERROR(10019),
		/** 拒绝写入外部SD卡 */
		WRITE_EXTERNAL_STORAGE_DENIED(10020),
		/** vid错误 */
		VID_ERROR(10021),
        /** 没有设置资源文件夹 */
        EXTRA_DIR_IS_NUll(10022),
        /** 不能创建资源目录 */
        NOT_CREATE_EXTRA_DIR(10023),
		/** 创建nomedia文件失败 */
		CREATE_NOMEDIA_ERROR(10024),
		/** 视频地址为空 */
		URL_IS_EMPTY(10025),
		/** 创建m3u8文件失败 */
		CREATE_M3U8_FILE_ERROR(10026),
		/** 写入m3u8文件失败 */
		WRITE_M3U8_FILE_ERROR(10027),
		/** 创建videojson文件失败 */
		CREATE_VIDEO_JSON_ERROR(10028),
		/** 写入videojson文件失败 */
		WRITE_VIDEO_JSON_ERROR(10029),
		/** 问答数据格式化成json错误 */
		QUESTION_FORMAT_JSON_ERROR(10030),
		/** 删除zip文件失败 */
		DELETE_ZIP_FILE_ERROR(10031),
		/** 创建视频临时文件错误 */
		CREATE_VIDEO_TMP_FILE_ERROR(10032),
		/** 下载视频文件长度错误 */
		DOWNLOAD_VIDEO_FILE_LENGTH_ERROR(10033),
		/** 视频http状态码错误*/
		VIDEO_HTTP_CODE_ERROR(10034),
		/** 视频文件下载错误 */
		VIDEO_DOWNLOAD_ERROR(10035),
		/** 视频重命名失败 */
		VIDEO_RENAME_ERROR(10036),
		/** 解压视频文件错误 */
		UNZIP_FILE_ERROR(10037),
		/** 创建临时压缩文件错误 */
		CREATE_ZIP_TMP_FILE_ERROR(10038),
		/** 创建解压目录错误 */
		CREATE_UNZIP_DIR_ERROR(10039),
		/** 下载压缩文件长度错误 */
		DOWNLOAD_ZIP_FILE_LENGTH_ERROR(10040),
		/** 压缩文件http状态码错误  */
		ZIP_HTTP_CODE_ERROR(10041),
		/** 压缩文件下载错误 */
		ZIP_DOWNLOAD_ERROR(10042),
		/** 压缩文件重命名失败 */
		ZIP_RENAME_ERROR(10043),
		/** 无法连接网络 */
		NETWORK_DENIED(10044);

		private final int code;
		ErrorType(int code) {
			this.code = code;
		}
		
		public int getCode() {
			return this.code;
		}
	}
}
