package com.easefun.polyvsdk.download.util;

import android.support.annotation.NonNull;

import com.easefun.polyvsdk.PolyvBitRate;
import com.easefun.polyvsdk.PolyvDownloader;
import com.easefun.polyvsdk.PolyvSDKClient;
import com.easefun.polyvsdk.PolyvSDKUtil;
import com.easefun.polyvsdk.Video;
import com.easefun.polyvsdk.database.question.QuestionDatabaseService;
import com.easefun.polyvsdk.database.video.VideoDatabaseService;
import com.easefun.polyvsdk.download.constant.PolyvDeleteVideoResult;
import com.easefun.polyvsdk.download.constant.PolyvDownloaderVideoStatus;
import com.easefun.polyvsdk.download.vo.PolyvDownloaderVideoVO;
import com.easefun.polyvsdk.video.PolyvValidateM3U8VideoReturnType;
import com.easefun.polyvsdk.video.PolyvVideoUtil;

import java.io.File;
import java.util.ArrayList;

/**
 * 下载工具类
 * @author Lion 2017-7-24
 */
public class PolyvDownloaderUtils {
    /**
     * 删除该视频所有文件，
     * 本方法是视频文件删除的工具方法，不涉及{@link PolyvDownloader} 实例，没有实例方法的停止功能，
     * 请在线程中执行该方法，避免卡顿和ANE。
     * @param videoId 视频id
     * @return {@link PolyvDeleteVideoResult}常量
     */
    @PolyvDeleteVideoResult.DeleteVideoResult
    public static int deleteVideo(@NonNull String videoId) {
        return deleteVideo(videoId, PolyvBitRate.ziDong.getNum());
    }

    /**
     * 删除该视频所有文件，
     * 本方法是视频文件删除的工具方法，不涉及{@link PolyvDownloader} 实例，没有实例方法的停止功能，
     * 请在线程中执行该方法，避免卡顿和ANE。
     * @param videoId 视频id
     * @param bitrate 码率（清晰度）
     * @return {@link PolyvDeleteVideoResult}常量
     */
    @PolyvDeleteVideoResult.DeleteVideoResult
    public static int deleteVideo(@NonNull String videoId, int bitrate) {
        if (!PolyvSDKUtil.validateVideoId(videoId)) {
            return PolyvDeleteVideoResult.VIDEO_ID_ERROR;
        }

        PolyvBitRate polyvBitRate = PolyvBitRate.getBitRate(bitrate);
        if (polyvBitRate == null) {
            return PolyvDeleteVideoResult.BITRATE_ERROR;
        }

        File downloadDir = PolyvSDKClient.getInstance().getDownloadDir();
        if (downloadDir == null) {
            return PolyvDeleteVideoResult.DOWNLOADER_DIR_NULL;
        }

        if (polyvBitRate == PolyvBitRate.ziDong) {
            int minBitRate = PolyvBitRate.getMinBitRate().getNum();
            int maxBitRate = PolyvBitRate.getMaxBitRate().getNum();
            for (int i = minBitRate; i <= maxBitRate; i++) {
                boolean value = deleteAllVideoFile(videoId, i);
                if (!value) {
                    return PolyvDeleteVideoResult.DELETE_VIDEO_FILE_FAIL;
                }
            }
        } else {
            boolean value = deleteAllVideoFile(videoId, bitrate);
            if (!value) {
                return PolyvDeleteVideoResult.DELETE_VIDEO_FILE_FAIL;
            }
        }

        boolean value = deleteDir(PolyvSDKClient.getInstance().getVideoDownloadExtraResourceDir(videoId));
        if (!value) {
            return PolyvDeleteVideoResult.DELETE_VIDEO_FILE_FAIL;
        }

        QuestionDatabaseService qusetionDBService = PolyvSDKClient.getInstance().getQuestionDBService();
        qusetionDBService.deleteQuestionByVid(videoId);
        VideoDatabaseService dbService = PolyvSDKClient.getInstance().getVideoDBService();
        dbService.deleteVideo(videoId);
        return PolyvDeleteVideoResult.DELETE_VIDEO_SUCCESS;
    }

    private static boolean deleteAllVideoFile(String vid, int bitRate) {
        String downloadDirPath = PolyvSDKClient.getInstance().getDownloadDir().getAbsolutePath();
        int range = vid.lastIndexOf("_");
        String vpid = vid.substring(0, range);
        boolean value;

        // 删除本地的m3u8文件
        String localm3u8 = downloadDirPath + File.separator + vpid + "_" + bitRate + ".m3u8";
        File localm3u8File = new File(localm3u8);
        if (localm3u8File.exists()) {
            value = localm3u8File.delete();
            if (!value) {
                return false;
            }
        }

        //删除本地的1.5Xm3u8文件
        String local15Xm3u8 = downloadDirPath + File.separator +
                Video.HlsSpeedType.SPEED_1_5X.getName() + "_" + vpid + "_" + bitRate + ".m3u8";
        File local15Xm3u8File = new File(local15Xm3u8);
        if (local15Xm3u8File.exists()) {
            value = local15Xm3u8File.delete();
            if (!value) {
                return false;
            }
        }

        // 删除本地的key文件
        String localkey = downloadDirPath + File.separator + vpid + "_" + bitRate + ".key";
        File localkeyFile = new File(localkey);
        if (localkeyFile.exists()) {
            value = localkeyFile.delete();
            if (!value) {
                return false;
            }
        }

        // 删除本地的1.5Xkey文件
        String local15Xkey = downloadDirPath + File.separator +
                Video.HlsSpeedType.SPEED_1_5X.getName() + "_" + vpid + "_" + bitRate + ".key";
        File local15XkeyFile = new File(local15Xkey);
        if (local15XkeyFile.exists()) {
            value = local15XkeyFile.delete();
            if (!value) {
                return false;
            }
        }

        //删除本地的json文件
        String localjson = downloadDirPath + File.separator + vpid + "_" + bitRate + ".json";
        File localjsonFile = new File(localjson);
        if (localjsonFile.exists()) {
            value = localjsonFile.delete();
            if (!value) {
                return false;
            }
        }

        //删除本地的1.5Xjson文件
        String local15Xjson = downloadDirPath + File.separator +
                Video.HlsSpeedType.SPEED_1_5X.getName() + "_" + vpid + "_" + bitRate + ".json";
        File local15XjsonFile = new File(local15Xjson);
        if (local15XjsonFile.exists()) {
            value = local15XjsonFile.delete();
            if (!value) {
                return false;
            }
        }

        // 删除本地的mp4文件
        String localmp4 = downloadDirPath + File.separator + vpid + "_" + bitRate + ".mp4";
        File localmp4File = new File(localmp4);
        if (localmp4File.exists()) {
            value = localmp4File.delete();
            if (!value) {
                return false;
            }
        }

        //删除本地文件夹或者临时文件
        String dir = downloadDirPath + File.separator + vpid + "_" + bitRate;
        File videoDir = new File(dir);
        if (videoDir.exists()) {
            if (videoDir.isDirectory()) {
                value = deleteDir(videoDir);
            } else {
                value = videoDir.delete();
            }

            if (!value) {
                return false;
            }
        }

        // 删除本地文件夹或者临时文件
        String localTmp = downloadDirPath + File.separator + vpid + "_" + bitRate;
        File localtmpFile = new File(localTmp);
        if (localtmpFile.exists()) {
            if (localtmpFile.isDirectory()) {
                value = deleteDir(localtmpFile);
            } else {
                value = localtmpFile.delete();
            }

            if (!value) {
                return false;
            }
        }

        // 删除本地其他后缀的视频文件
        File videoFile = PolyvVideoUtil.validateVideo(vid, bitRate);
        if (videoFile != null) {
            value = videoFile.delete();
            if (!value) {
                return false;
            }
        }

        //删除1.5文件夹
        String dir15x = downloadDirPath + File.separator +
                Video.HlsSpeedType.SPEED_1_5X.getName() + "_" + vpid + "_" + bitRate;
        File videoDir15x = new File(dir15x);
        value = deleteDir(videoDir15x);
        return value;
    }

    /**
     * 获取下载目录下载视频列表
     * @return 下载视频列表
     */
    @NonNull
    public static ArrayList<PolyvDownloaderVideoVO> getDownloadDirVideoList() {
        return getDownloadDirVideoList(PolyvSDKClient.getInstance().getDownloadDir());
    }

    /**
     * 获取目录中下载视频列表
     * @param downloadDir 目录
     * @return 下载视频列表
     */
    @NonNull
    public static ArrayList<PolyvDownloaderVideoVO> getDownloadDirVideoList(File downloadDir) {
        if (downloadDir == null) {
            return new ArrayList<>();
        }

        if (!downloadDir.isDirectory()) {
            return new ArrayList<>();
        }

        if (!downloadDir.exists()) {
            return new ArrayList<>();
        }

        File[] files = downloadDir.listFiles();
        if (files == null || files.length == 0) {
            return new ArrayList<>();
        }

        //验证视频文件
        ArrayList<PolyvDownloaderVideoVO> downloaderVideoVOs = new ArrayList<>();
        String fileName, vpbid;
        VideoInfo videoInfo;
        for (File file : files) {
            if (file.isDirectory()) {
                //2.0 m3u8的存储结构vpbid/vpbid.m3u8
                fileName = file.getName();
                //判断目录名称是否是videoPoolBitrate的结构
                if (PolyvSDKUtil.validateVideoPoolBitrateId(fileName)) {
                    vpbid = fileName;
                    videoInfo = getVideoInfo(vpbid);
                    int validateResult = PolyvVideoUtil.validateM3U8Video(videoInfo.getVideoId(), videoInfo.getBitrate());
                    switch (validateResult) {
                        case PolyvValidateM3U8VideoReturnType.M3U8_CORRECT:
                            downloaderVideoVOs.add(new PolyvDownloaderVideoVO(videoInfo.getVideoId(), videoInfo.getBitrate(), PolyvDownloaderVideoStatus.VIDEO_CORRECT));
                            break;
                        case PolyvValidateM3U8VideoReturnType.M3U8_FILE_NOT_FOUND:
                            break;
                        case PolyvValidateM3U8VideoReturnType.M3U8_FILE_CONTENT_EMPTY:
                        case PolyvValidateM3U8VideoReturnType.M3U8_KEY_FILE_NOT_FOUND:
                        case PolyvValidateM3U8VideoReturnType.M3U8_TS_FILE_NOT_FOUND:
                        case PolyvValidateM3U8VideoReturnType.M3U8_TS_LIST_EMPTY:
                            downloaderVideoVOs.add(new PolyvDownloaderVideoVO(videoInfo.getVideoId(), videoInfo.getBitrate(), PolyvDownloaderVideoStatus.VIDEO_INCOMPLETE));
                            break;
                    }
                }

                continue;
            }

            //过滤key,json文件
            fileName = file.getName();
            if (fileName.endsWith(".key") || fileName.endsWith(".json")) {
                continue;
            }

            //获取文件名前缀
            int index = fileName.lastIndexOf(".");
            if (index != -1) {
                vpbid = fileName.substring(0, index);
            } else {
                vpbid = fileName;
            }

            //判断是否符合videoPoolBitrateId规范
            if (!PolyvSDKUtil.validateVideoPoolBitrateId(vpbid)) {
                continue;
            }

            //判断是否是没有后缀的文件，是下载视频的临时文件，表示没有下载完整的视频文件。
            videoInfo = getVideoInfo(vpbid);
            if (!fileName.contains(".")) {
                File tmpVideo = PolyvVideoUtil.validateTmpVideo(videoInfo.getVideoId(), videoInfo.getBitrate());
                if (tmpVideo != null) {
                    downloaderVideoVOs.add(new PolyvDownloaderVideoVO(videoInfo.getVideoId(), videoInfo.getBitrate(), PolyvDownloaderVideoStatus.VIDEO_INCOMPLETE));
                    continue;
                }
            }

            //正常有后缀的视频文件
            if (fileName.endsWith(".m3u8")) {
                //1.0 m3u8存储结构vpbid.m3u8
                int validateResult = PolyvVideoUtil.validateM3U8Video(videoInfo.getVideoId(), videoInfo.getBitrate());
                switch (validateResult) {
                    case PolyvValidateM3U8VideoReturnType.M3U8_CORRECT:
                        downloaderVideoVOs.add(new PolyvDownloaderVideoVO(videoInfo.getVideoId(), videoInfo.getBitrate(), PolyvDownloaderVideoStatus.VIDEO_CORRECT));
                        break;
                    case PolyvValidateM3U8VideoReturnType.M3U8_FILE_NOT_FOUND:
                        break;
                    case PolyvValidateM3U8VideoReturnType.M3U8_FILE_CONTENT_EMPTY:
                    case PolyvValidateM3U8VideoReturnType.M3U8_KEY_FILE_NOT_FOUND:
                    case PolyvValidateM3U8VideoReturnType.M3U8_TS_FILE_NOT_FOUND:
                    case PolyvValidateM3U8VideoReturnType.M3U8_TS_LIST_EMPTY:
                        downloaderVideoVOs.add(new PolyvDownloaderVideoVO(videoInfo.getVideoId(), videoInfo.getBitrate(), PolyvDownloaderVideoStatus.VIDEO_INCOMPLETE));
                        break;
                }
            } else if (fileName.endsWith(".mp4")) {
                File mp4Video  = PolyvVideoUtil.validateMP4Video(videoInfo.getVideoId(), videoInfo.getBitrate());
                if (mp4Video != null) {
                    downloaderVideoVOs.add(new PolyvDownloaderVideoVO(videoInfo.getVideoId(), videoInfo.getBitrate(), PolyvDownloaderVideoStatus.VIDEO_CORRECT));
                }
            } else {
                File video = PolyvVideoUtil.validateVideo(videoInfo.getVideoId(), videoInfo.getBitrate());
                if (video != null) {
                    downloaderVideoVOs.add(new PolyvDownloaderVideoVO(videoInfo.getVideoId(), videoInfo.getBitrate(), PolyvDownloaderVideoStatus.VIDEO_CORRECT));
                }
            }
        }

        //验证有哪些m3u8视频有保存目录却没有m3u8文件，表示m3u8视频文件没有下载完成。
        ArrayList<String> verifiedVpbIdList = new ArrayList<>(downloaderVideoVOs.size());
        String vpid;
        for (PolyvDownloaderVideoVO downloaderVideoVO : downloaderVideoVOs) {
            vpid = downloaderVideoVO.getVideoId().substring(0, downloaderVideoVO.getVideoId().lastIndexOf("_"));
            verifiedVpbIdList.add(vpid + "_" + downloaderVideoVO.getBitrate());
        }

        for (File file : files) {
            if (!file.isDirectory()) {
                continue;
            }

            vpbid = file.getName();
            if (!PolyvSDKUtil.validateVideoPoolBitrateId(vpbid)) {
                continue;
            }

            //没有被上面验证视频文件逻辑过的
            if (verifiedVpbIdList.contains(vpbid)) {
                continue;
            }

            videoInfo = getVideoInfo(vpbid);
            downloaderVideoVOs.add(new PolyvDownloaderVideoVO(videoInfo.getVideoId(), videoInfo.getBitrate(), PolyvDownloaderVideoStatus.VIDEO_INCOMPLETE));
        }

        return downloaderVideoVOs;
    }

    private static VideoInfo getVideoInfo(String vpbid) {
        String vid = vpbid.substring(0, vpbid.lastIndexOf("_") + 1) + vpbid.substring(0, 1);
        int bitrate = Integer.parseInt(vpbid.substring(vpbid.length() - 1));
        return new VideoInfo(vid, bitrate);
    }

    /**
     * 删除整个下载目录
     * @return {@code true}:删除成功<br/>{@code false}:删除失败
     */
    public static boolean deleteDownloaderDir() {
        File downloadDir = PolyvSDKClient.getInstance().getDownloadDir();
        return downloadDir == null || deleteDir(downloadDir);
    }

    /**
     * 删除参数的文件或者递归删除目录下的所有文件。
     * 请在线程中执行该方法，避免卡顿和ANE。
     * @param dir 文件或者目录
     * @return {@code true}:删除成功<br/>{@code false}:删除失败
     */
    public static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();// 递归删除目录中的子目录下
            if (children != null) {
                for (String aChildren : children) {
                    boolean success = deleteDir(new File(dir, aChildren));
                    if (!success) {
                        return false;
                    }
                }
            }
        }

        if (dir.exists()) {
            return dir.delete();
        }

        return true;
    }

    private static class VideoInfo {
        private final String videoId;
        private final int bitrate;

        VideoInfo(String videoId, int bitrate) {
            this.videoId = videoId;
            this.bitrate = bitrate;
        }

        String getVideoId() {
            return videoId;
        }

        int getBitrate() {
            return bitrate;
        }
    }
}
