package com.easefun.polyvsdk.upload;


import android.content.Context;

public interface IPolyvUploader {

    /**
     * 开始上传
     */
    void start();

    /**
     * 开始上传，该方法会验证用户剩余空间是否可用。
     *
     * @param sign  签名，生成规则为：SHA1("ptime="+ptime+secretKey).toUpperCase()，ptime为当前时间的毫秒级时间戳（13位），3分钟内有效。或者使用{@link PolyvUploader#getSign(String, long)}获取。
     * @param ptime 生成签名的时间戳
     */
    void start(String sign, long ptime);

    /**
     * 暂停上传
     */
    void pause();

    /**
     * 是否在上传中
     *
     * @return
     */
    boolean isUploading();

    /**
     * 如果使用分块上传，则需要设置上下文，否则请不要设置
     *
     * @param context 需为activity实例
     */
    void setContext(Context context);

    /**
     * 如果使用分块上传，则需要在删除任务的时候，删除信息文件
     *
     * @param context
     */
    void deleteInfoFile(Context context);

    /**
     * 如果使用分块上传，可以设置信息文件的保存路径
     *
     * @param absolutePath
     */
    void setInfoFilePath(String absolutePath);

    /**
     * 设置监听器
     *
     * @param uploadListener
     */
    void setUploadListener(PolyvUploader.UploadListener uploadListener);

}
