package com.easefun.polyvsdk.upload;

import android.content.Context;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class PolyvUploaderManager {
    private static final Map<String, PolyvUploader> polyvUploaderMap = new ConcurrentHashMap<>();
    // private static final Map<String, String> vidMap = new
    // ConcurrentHashMap<String, String>();
    // private static ExecutorService executor;

    /**
     * 获取上传器
     *
     * @param filePath 文件路径
     * @param title    标题
     * @param desc     描述
     * @return 上传器
     */
    public static PolyvUploader getPolyvUploader(final String filePath, final String title, final String desc) {
        return getPolyvUploader(filePath, title, desc, "0");
    }

    /**
     * 获取上传器
     *
     * @param filePath 文件路径
     * @param title    标题
     * @param desc     描述
     * @param cataid   上传的分类目录id
     * @return 上传器
     */
    public static PolyvUploader getPolyvUploader(final String filePath, final String title, final String desc, String cataid) {
        PolyvUploader polyvuploader = polyvUploaderMap.get(filePath + "_" + cataid);
        if (polyvuploader == null) {
            polyvuploader = new PolyvUploader(filePath, title, desc, cataid);
            polyvUploaderMap.put(filePath + "_" + cataid, polyvuploader);
        }
        return polyvuploader;
        // String vid = vidMap.get(filePath);
        // if (vid != null) {
        // PolyvMTUploadVideo polyvDownloader = (PolyvMTUploadVideo)
        // polyvDownloaderMap.get(vid);
        // if (polyvDownloader == null) {
        // polyvDownloader = new PolyvMTUploadVideo(filePath, title, desc);
        // polyvDownloaderMap.put(vid, polyvDownloader);
        // }
        // return polyvDownloader;
        // } else {
        // if (executor == null)
        // executor = Executors.newCachedThreadPool();
        // Future<PolyvMTUploadVideo> result = executor.submit(new
        // Callable<PolyvMTUploadVideo>() {
        // @Override
        // public PolyvMTUploadVideo call() throws Exception {
        // String vid = vidMap.get(filePath);
        // if (vid == null) {
        // vid = PolyvMTUploadVideo.getVid(filePath);
        // vidMap.put(filePath, vid);
        // }
        // PolyvMTUploadVideo polyvDownloader = (PolyvMTUploadVideo)
        // polyvDownloaderMap.get(vid);
        // if (polyvDownloader == null) {
        // polyvDownloader = new PolyvMTUploadVideo(filePath, title, desc);
        // polyvDownloaderMap.put(vid, polyvDownloader);
        // }
        // return polyvDownloader;
        // }
        //
        // });
        // try {
        // return result.get();
        // } catch (InterruptedException e) {
        // } catch (ExecutionException e) {
        // } finally {
        // if (executor != null) {
        // executor.shutdown();
        // executor = null;
        // }
        // }
        // return null;
        // }
    }

    public static int getSize() {
        return polyvUploaderMap.size();
    }

    // private static MTDownloadVideo clearPolyvDownload(String vid, int
    // bitRate) {
    // StringBuilder key = new StringBuilder();
    // key.append(vid).append("_").append(bitRate);
    // MTDownloadVideo polyvDownloader = (MTDownloadVideo)
    // polyvDownloaderMap.get(key.toString());
    // if (polyvDownloader != null) {
    // polyvDownloader.setDownloadListener(null);
    // polyvDownloader.pause();
    // }
    //
    // return polyvDownloader;
    // }

    /**
     * 移除上传任务
     *
     * @param filePath
     */
    public static IPolyvUploader removePolyvUpload(String filePath) {
        // String vid = vidMap.get(filePath);
        // if (vid != null && polyvDownloaderMap.containsKey(vid)) {
        // polyvDownloaderMap.remove(vid);
        // }
        if (polyvUploaderMap.containsKey(filePath)) {
            polyvUploaderMap.get(filePath).pause();
            polyvUploaderMap.get(filePath).setUploadListener(null);
            return polyvUploaderMap.remove(filePath);
        }
        return null;
    }

    /**
     * 开始所有上传任务
     *
     * @param context
     */
    public static void startAll(Context context) {
        for (Map.Entry<String, PolyvUploader> entry : polyvUploaderMap.entrySet()) {
            IPolyvUploader polyvMTUploadVideo = entry.getValue();
            polyvMTUploadVideo.setContext(context);
            polyvMTUploadVideo.start();
        }
    }

    /**
     * 开始所有上传任务
     */
    public static void startAll() {
        for (Map.Entry<String, PolyvUploader> entry : polyvUploaderMap.entrySet()) {
            entry.getValue().start();
        }
    }

    /**
     * 开始未完成的任务
     *
     * @param finishKeys 完成任务的key集合，key=filepath
     */
    public static void startUnfinished(List<String> finishKeys) {
        for (Map.Entry<String, PolyvUploader> entry : polyvUploaderMap.entrySet()) {
            boolean flag = false;
            if (finishKeys != null && !finishKeys.isEmpty()) {
                for (String key : finishKeys) {
                    if (key.equals(entry.getKey())) {
                        flag = true;
                        break;
                    }
                }
            }
            if (flag)
                continue;
            entry.getValue().start();
        }
    }

    /**
     * 停止所有上传任务
     */
    public static void stopAll() {
        for (Map.Entry<String, PolyvUploader> entry : polyvUploaderMap.entrySet())
            entry.getValue().pause();
    }
}
